import argparse
import ast
from appdirs import *
from test_junkie.settings import Settings
from test_junkie.cli.cli import CliUtils


class ConfigManager:
    """
    only use for cli, do not use for parsing the config when running tests
    """
    __CONFIG_NAME = "tj.cfg"
    __DEFAULTS = """
[runtime]
test_multithreading_limit=None
suite_multithreading_limit=None
html_report=None
xml_report=None
monitor_resources=None
tests=None
features=None
components=None
owners=None
run_on_match_all=None
run_on_match_any=None
skip_on_match_all=None
skip_on_match_any=None
sources=None
"""

    def __init__(self, command=None, args=None):

        self.path = ConfigManager.get_config_path()
        self.args = args

        if command is not None and args is not None:
            if not os.path.exists(self.path):
                self.restore(new=True)
            self.config = ConfigManager.get_config_parser(self.path)
            getattr(self, command)()

    @staticmethod
    def get_root_dir():
        return user_data_dir("Test-Junkie")

    @staticmethod
    def get_config_path():
        return "{root}{sep}{name}".format(root=ConfigManager.get_root_dir(),
                                          name=ConfigManager.__CONFIG_NAME,
                                          sep=os.sep)

    @staticmethod
    def get_config_parser(path):
        if sys.version_info[0] < 3:
            # Python 2
            import ConfigParser as configparser
        else:
            # Python 3, module was renamed to configparser
            import configparser
        config = configparser.ConfigParser()
        config.read(path)
        return config

    def __set_default_config(self):

        with open(self.path, "w+") as doc:
            doc.write(ConfigManager.__DEFAULTS)
            print("[{status}] Config restored to default settings!"
                  .format(status=CliUtils.format_color_string(value="OK", color="green")))

    def __restore_value(self, option, value):

        try:
            self.config.set('runtime', option, str(value))
            with open(self.path, 'w+') as doc:
                self.config.write(doc)
            print("[{status}]\t{option}={value}".format(
                status=CliUtils.format_color_string(value="OK", color="green"), option=option, value=value))
        except:
            print("[{status}]\tUnexpected error occurred during update of {setting}={value}"
                  .format(status=CliUtils.format_color_string(value="ERROR", color="red"), setting=option, value=value))
            CliUtils.print_color_traceback()
            exit(120)

    def __print_value(self, option):

        if option in self.config.options("runtime"):
            value = ConfigManager.get_value(self.config, option)
            if value != Settings.UNDEFINED:
                print("{option}={value}".format(option=option, value=ast.literal_eval(value)))

    @staticmethod
    def get_value(config, option):

        if sys.version_info[0] < 3:
            # Python 2
            value = config.get("runtime", option, Settings.UNDEFINED)
        else:
            # Python 3, module is not backwards compatible and fallback has to be explicitly assigned
            value = config.get("runtime", option, fallback=Settings.UNDEFINED)
        return value

    def update(self):

        parser = argparse.ArgumentParser(description="Update configuration settings for individual properties",
                                         usage="tj config update [OPTIONS]")
        CliUtils.add_standard_tj_args(parser)
        args = parser.parse_args(self.args[3:])
        if not self.args[3:]:
            print("[{status}]\tWhat do you want to update?\n".format(
                status=CliUtils.format_color_string(value="ERROR", color="red")))
            parser.print_help()
            return
        for option, value in args.__dict__.items():
            if value is not Settings.UNDEFINED:
                self.__restore_value(option, value)
        print("[{status}]\tRestore original value with tj config restore [OPTION]. tj config restore -h for more info."
              .format(status=CliUtils.format_color_string(value="TIP", color="blue")))

    def show(self):

        parser = argparse.ArgumentParser(description='Display current configuration for Test-Junkie',
                                         usage="tj config show [OPTIONS]")
        parser.add_argument("-a", "--all", action="store_true", default=False,
                            help="Will restore all config settings to its default values")
        CliUtils.add_standard_boolean_tj_args(parser)
        args = parser.parse_args(self.args[3:])
        if args.all or not self.args[3:]:
            print("Config is located at: {path}\n"
                  .format(path=CliUtils.format_color_string(value=self.path, color="green")))
            with open(self.path, "r") as cfg:
                print(cfg.read())
            return
        for option, value in args.__dict__.items():
            if value is True:
                self.__print_value(option)

    def restore(self, new=False):

        parser = argparse.ArgumentParser(description='Restore config settings to it\'s original values',
                                         usage="tj config restore [OPTIONS]")
        parser.add_argument("-a", "--all", action="store_true", default=False,
                            help="Will restore all config settings to its default values")
        CliUtils.add_standard_boolean_tj_args(parser)
        args = parser.parse_args(self.args[3:])
        if args.all or new:
            if not os.path.exists(ConfigManager.get_root_dir()):
                os.makedirs(ConfigManager.get_root_dir())
            if os.path.exists(self.path):
                os.remove(self.path)
            self.__set_default_config()
            return
        if not self.args[3:]:
            print("[{status}]\tWhat do you want to restore?\n".format(
                status=CliUtils.format_color_string(value="ERROR", color="red")))
            parser.print_help()
            return
        for option, value in args.__dict__.items():
            if value is True:
                self.__restore_value(option, None)



import argparse
import pprint
import sys
import traceback
import pkg_resources

from test_junkie.builder import Builder
from test_junkie.cli.cli_aggregator import CliAggregator
from test_junkie.constants import DocumentationLinks
from test_junkie.errors import BadCliParameters
from test_junkie.settings import Settings
from colorama import Fore, Style


class Cli(object):

    def __init__(self):
        parser = argparse.ArgumentParser(
            description="",
            usage="""tj COMMAND

Modern Testing Framework

Commands:
run\t Run tests in any directory (recursive) 
audit\t Audit your tests in any directory (recursive) 
config\t Configure Test Junkie
version\t Display current version

Use: tj COMMAND -h to display COMMAND specific help
""")
        parser.add_argument('command', help='command to run')
        args = parser.parse_args(sys.argv[1:2])
        if not hasattr(self, args.command):
            print("[{status}]\t\'{command}\' is not a test-junkie command\n".format(
                status=CliUtils.format_color_string(value="ERROR", color="red"), command=args.command))
            parser.print_help()
            exit(1)
        if args.command:
            getattr(self, args.command)()

    def run(self):
        parser = argparse.ArgumentParser(description='Run tests from command line',
                                         usage="tj run [OPTIONS]")

        parser.add_argument("-x", "--suites", nargs="+", default=None,
                            help="Test Junkie will only run suites provided, "
                                 "given that they are found in the SOURCE")

        parser.add_argument("-v", "--verbose", action="store_true", default=False,
                            help="Enables Test Junkie's logs for debugging purposes")

        CliUtils.add_standard_tj_args(parser)

        args = parser.parse_args(sys.argv[2:])

        if args.verbose:
            from test_junkie.debugger import LogJunkie
            LogJunkie.enable_logging(10)

        from test_junkie.cli.cli_runner import RunnerManager
        try:
            tj = RunnerManager(sources=args.sources, ignore=[".git"], suites=args.suites)
            tj.scan()
        except BadCliParameters as error:
            print("[{status}] {error}".format(status=CliUtils.format_color_string("ERROR", "red"), error=error))
            return
        tj.run_suites(args)

    def audit(self):
        parser = argparse.ArgumentParser(usage="""tj audit [OPTIONS]

Scan and display aggregated and/or filtered test information
""")

        parser.add_argument("--by-tags", action="store_true", default=False,
                            help="Present aggregated data broken down by tags")

        parser.add_argument("--by-suites", action="store_true", default=False,
                            help="Present aggregated data broken down by suites")

        parser.add_argument("--by-owners", action="store_true", default=False,
                            help="Present aggregated data broken down by owners")

        parser.add_argument("--by-components", action="store_true", default=False,
                            help="Present aggregated data broken down by components")

        parser.add_argument("--by-features", action="store_true", default=False,
                            help="Present aggregated data broken down by features")

        parser.add_argument("--no-rules", action="store_true", default=False,
                            help="Aggregate data only for suites that do not have any rules set")

        parser.add_argument("--no-listeners", action="store_true", default=False,
                            help="Aggregate data only for suites that do not have any event listeners set")

        parser.add_argument("--no-suite-retries", action="store_true", default=False,
                            help="Aggregate data only for suites that do not have retries set")

        parser.add_argument("--no-test-retries", action="store_true", default=False,
                            help="Aggregate data only for tests that do not have retries set")

        parser.add_argument("--no-suite-meta", action="store_true", default=False,
                            help="Aggregate data only for suites that do not have any meta information set")

        parser.add_argument("--no-test-meta", action="store_true", default=False,
                            help="Aggregate data only for tests that do not have any meta information set")

        parser.add_argument("--no-owners", action="store_true", default=False,
                            help="Aggregate data only for tests that do not have any owners defined")

        parser.add_argument("--no-features", action="store_true", default=False,
                            help="Aggregate data only for suites that do not have features defined")

        parser.add_argument("--no-components", action="store_true", default=False,
                            help="Aggregate data only for tests that do not have any components defined")

        parser.add_argument("-x", "--suites", nargs="+", default=None,
                            help="Test Junkie will only run suites provided, "
                                 "given that they are found in the SOURCE")

        parser.add_argument("-v", "--verbose", action="store_true", default=False,
                            help="Enables Test Junkie's logs for debugging purposes")

        CliUtils.add_standard_tj_args(parser, audit=True)
        args = parser.parse_args(sys.argv[2:])

        if args.verbose:
            from test_junkie.debugger import LogJunkie
            LogJunkie.enable_logging(10)

        from test_junkie.cli.cli_runner import RunnerManager
        try:
            tj = RunnerManager(sources=args.sources, ignore=[".git"], suites=args.suites)
            tj.scan()
        except BadCliParameters as error:
            print("[{status}] {error}".format(status=CliUtils.format_color_string("ERROR", "red"), error=error))
            return
        aggregator = CliAggregator(suites=tj.suites, args=args)
        aggregator.aggregate()
        aggregator.print_results()

    def config(self):
        parser = argparse.ArgumentParser(usage="""tj config COMMAND

Allows to configure Test Junkie the way you want it

Commands:
show\t Display current configuration for Test-Junkie
update\t Update configuration settings for individual properties via cli 
restore\t Will restore config to it\'s original values

Use: tj config COMMAND -h to display COMMAND specific help
""")
        parser.add_argument('command', default=None, help="command to run")
        try:
            if len(sys.argv) >= 3:
                command = str(sys.argv[2:3][0])
                if command in ["show", "update", "restore"]:
                    from test_junkie.cli.cli_config import ConfigManager
                    return ConfigManager(command, sys.argv)
            parser.print_help()
        except:
            if "SystemExit:" not in traceback.format_exc():
                CliUtils.print_color_traceback()
                parser.print_help()
                exit(120)

    def version(self):
        print("Test Junkie {} (Python{})".format(pkg_resources.require("test-junkie")[0].version, sys.version_info[0]))


class CliUtils:

    def __init__(self):

        pass

    @staticmethod
    def add_standard_tj_args(parser, audit=False):
        """
        Generic parser args used to configure execution or set config settings
        """
        if not audit:
            parser.add_argument("-T", "--test_multithreading_limit", type=int, default=Settings.UNDEFINED,
                                help="Test level multi threading allows to run multiple tests concurrently.")

            parser.add_argument("-S", "--suite_multithreading_limit", type=int, default=Settings.UNDEFINED,
                                help="Suite level multi threading allows to run multiple suites concurrently.")

            parser.add_argument("-t", "--tests", nargs="+", default=Settings.UNDEFINED,
                                help="Test Junkie can run specific tests. "
                                     "Provide the names of the tests that you want to execute/audit.")

        parser.add_argument("-f", "--features", nargs="+", default=Settings.UNDEFINED,
                            help="Test suites can be defined with a feature that they are testing. "
                                 "Use features to narrow down execution/audit of test suites only to those that "
                                 "match this filter. Learn more @ {link}".format(link=DocumentationLinks.FEATURES))

        parser.add_argument("-c", "--components", nargs="+", default=Settings.UNDEFINED,
                            help="Tests can be defined with a component that they are testing. "
                                 "Use components to narrow down execution/audit of tests only to those that "
                                 "match this filter. Learn more @ {link}".format(link=DocumentationLinks.COMPONENTS))

        parser.add_argument("-o", "--owners", nargs="+", default=Settings.UNDEFINED,
                            help="Tests & test suites can be defined with an assignee. "
                                 "Use owners to narrow down execution/audit of tests only to those that "
                                 "match this filter. Learn more @ {link}".format(link=DocumentationLinks.ASSIGNEES))

        if not audit:
            parser.add_argument("-m", "--monitor_resources", action="store_true", default=Settings.UNDEFINED,
                                help="Test Junkie can track resource usage for CPU & Memory as it runs tests")

            parser.add_argument("--html_report", type=str, default=Settings.UNDEFINED,
                                help="Path to FILE. This will enable HTML report generation and when ready, "
                                     "the report will be saved to the specified file")

            parser.add_argument("--xml_report", type=str, default=Settings.UNDEFINED,
                                help="Path to FILE. This will enable XML report generation and when ready, "
                                     "the report will be saved to the specified file")

            parser.add_argument("-l", "--run_on_match_all", nargs="+", default=Settings.UNDEFINED,
                                help="Test Junkie will RUN tests that match ALL of the tags. Read more about it: {link}"
                                .format(link=DocumentationLinks.TAGS))

            parser.add_argument("-k", "--run_on_match_any", nargs="+", default=Settings.UNDEFINED,
                                help="Test Junkie will RUN tests that match ANY of the tags. Read more about it: {link}"
                                .format(link=DocumentationLinks.TAGS))

            parser.add_argument("-j", "--skip_on_match_all", nargs="+", default=Settings.UNDEFINED,
                                help="Test Junkie will SKIP tests that match ALL of the tags. Read more about it: {link}"
                                .format(link=DocumentationLinks.TAGS))

            parser.add_argument("-g", "--skip_on_match_any", nargs="+", default=Settings.UNDEFINED,
                                help="Test Junkie will SKIP tests that match ANY of the tags. Read more about it: {link}"
                                .format(link=DocumentationLinks.TAGS))
        else:
            parser.add_argument("-l", "--tags", nargs="+", default=Settings.UNDEFINED,
                                help="Test Junkie will audit tests that match those tags.")

        parser.add_argument("-s", "--sources", nargs="+", default=Settings.UNDEFINED,
                            help="Paths to DIRECTORY or FILE where you have your tests. "
                                 "Test Junkie will traverse this source(s) looking for test suites")

    @staticmethod
    def add_standard_boolean_tj_args(parser):
        """
        Generic parser args used to show and restore config settings
        """

        parser.add_argument("-s", "--sources", action="store_true", default=False,
                            help="Paths to DIRECTORY or FILE where you have your tests. "
                                 "Test Junkie will traverse this source(s) looking for test suites")

        parser.add_argument("-T", "--test_multithreading_limit", action="store_true", default=False,
                            help="Test level multi threading allows to run multiple tests concurrently.")

        parser.add_argument("-S", "--suite_multithreading_limit", action="store_true", default=False,
                            help="Suite level multi threading allows to run multiple suites concurrently.")

        parser.add_argument("-t", "--tests", nargs="+", default=Settings.UNDEFINED,
                            help="Test Junkie can run specific tests. "
                                 "Provide the names of the tests that you want to run.")

        parser.add_argument("-f", "--features", action="store_true", default=False,
                            help="Test suites can be defined with a feature that they are testing. "
                                 "Use features to narrow down execution of test suites only to those that "
                                 "match this filter. Learn more @ {link}".format(link=DocumentationLinks.FEATURES))

        parser.add_argument("-c", "--components", action="store_true", default=False,
                            help="Tests can be defined with a component that they are testing. "
                                 "Use components to narrow down execution of tests only to those that "
                                 "match this filter. Learn more @ {link}".format(link=DocumentationLinks.COMPONENTS))

        parser.add_argument("-o", "--owners", action="store_true", default=False,
                            help="Tests & test suites can be defined with an assignee. "
                                 "Use owners to narrow down execution of tests only to those that "
                                 "match this filter. Learn more @ {link}".format(link=DocumentationLinks.ASSIGNEES))

        parser.add_argument("-m", "--monitor_resources", action="store_true", default=False,
                            help="Test Junkie can track resource usage for CPU & Memory as it runs tests")

        parser.add_argument("--html_report", action="store_true", default=False,
                            help="Path to FILE. This will enable HTML report generation and when ready, "
                                 "the report will be saved to the specified file")

        parser.add_argument("--xml_report", action="store_true", default=False,
                            help="Path to FILE. This will enable XML report generation and when ready, "
                                 "the report will be saved to the specified file")

        parser.add_argument("-l", "--run_on_match_all", action="store_true", default=False,
                            help="Test Junkie will RUN tests that match ALL of the tags. Read more about it: {link}"
                            .format(link=DocumentationLinks.TAGS))

        parser.add_argument("-k", "--run_on_match_any", action="store_true", default=False,
                            help="Test Junkie will RUN tests that match ANY of the tags. Read more about it: {link}"
                            .format(link=DocumentationLinks.TAGS))

        parser.add_argument("-j", "--skip_on_match_all", action="store_true", default=False,
                            help="Test Junkie will SKIP tests that match ALL of the tags. Read more about it: {link}"
                            .format(link=DocumentationLinks.TAGS))

        parser.add_argument("-g", "--skip_on_match_any", action="store_true", default=False,
                            help="Test Junkie will SKIP tests that match ANY of the tags. Read more about it: {link}"
                            .format(link=DocumentationLinks.TAGS))

    @staticmethod
    def format_color_string(value, color):
        colors = {"red": Fore.RED, "green": Fore.GREEN, "yellow": Fore.YELLOW, "blue": Fore.BLUE}
        return "{style}{color}{value}{reset}".format(style=Style.BRIGHT, color=colors[color],
                                                     value=value, reset=Style.RESET_ALL)

    @staticmethod
    def print_color_traceback():
        print(Style.BRIGHT + Fore.RED)
        print(traceback.format_exc())
        print(Style.RESET_ALL)


if "__main__" == __name__:

    Cli()



import subprocess
import sys
import traceback


class Cmd:

    def __init__(self):

        pass

    @staticmethod
    def run(cmd):

        try:
            if sys.version_info[0] < 3:
                output = str(subprocess.check_output(cmd))\
                    .replace("\r", "")\
                    .replace("\t", "")\
                    .strip()\
                    .split("\n")
            else:
                output = str(subprocess.check_output(cmd)) \
                    .replace("\\r", "") \
                    .replace("\\t", "") \
                    .strip() \
                    .split("\\n")
            for i in list(output):
                if i == "" or len(i) < 2:
                    output.remove(i)
            return output
        except:
            raise Exception("Command: {} produced exception: {}".format(cmd, traceback.format_exc()))



from test_junkie.cli.cli import Cli


def main():

    Cli()


"""
0. support test coverage!
1. Runner to take config path
2. .resources to save in designated directory or to specific file
3. limits to reference the config if config was set
4. cancel to return when suites are canceled
5. save html & xml to a designated directory or specific file
6. test cov for the new modules
7. Tests needs to be filtered on string or object!!!
"""
